@@ -1,7 +1,7 @@
 package Test::Script;
 
 # ABSTRACT: Basic cross-platform tests for scripts
-our $VERSION = '1.11_01'; # TRIAL VERSION
+our $VERSION = '1.11_02'; # TRIAL VERSION
 
 
 use 5.006;
@@ -14,6 +14,8 @@ use File::Spec::Unix ();
 use Probe::Perl      ();
 use IPC::Run3        qw( run3 );
 use Test::Builder    ();
+use File::Temp       ();
+use File::Path       ();
 
 our @ISA     = 'Exporter';
 our @EXPORT  = qw{
@@ -89,7 +91,8 @@ sub script_compiles {
   my $unix   = shift @$args;
   my $path   = path( $unix );
   my @libs   = map { "-I$_" } grep {!ref($_)} @INC;
-  my $cmd    = [ perl, @libs, '-c', $path, @$args ];
+  my $dir    = _preload_module();
+  my $cmd    = [ perl, "-I$dir", '-M__TEST_SCRIPT__', '-c', $path, @$args ];
   my $stdin  = '';
   my $stdout = '';
   my $stderr = '';
@@ -101,6 +104,8 @@ sub script_compiles {
     $error eq '' and $rv and $exit == 0 and $signal == 0 and $stderr =~ /syntax OK\s+\z/si
   );
 
+  File::Path::rmtree($dir);
+
   my $test = Test::Builder->new;
   $test->ok( $ok, $_[0] || "Script $unix compiles" );
   $test->diag( "$exit - $stderr" ) unless $ok;
@@ -110,6 +115,27 @@ sub script_compiles {
   return $ok;
 }
 
+# this is noticably slower for long @INC lists (sometimes present in cpantesters
+# boxes) than the previous implementation, which added a -I for every element in
+# @INC.  (also slower for more reasonable @INCs, but not noticably).  But it is
+# safer as very long argument lists can break calls to system
+sub _preload_module
+{
+  my $dir = File::Temp::tempdir( CLEANUP => 1 );
+  # this is hopefully a pm file that nobody would use
+  my $filename = File::Spec->catfile($dir, '__TEST_SCRIPT__.pm');
+  my $fh;
+  open($fh, ">$filename");
+  print $fh 'unshift @INC, ',
+    join ',', 
+    # quotemeta is overkill, but it will make sure that characters
+    # like " are quoted
+    map { '"' . quotemeta . '"' }
+    grep { ! ref } @INC;
+  close $fh;
+  $dir;
+}
+
 
 my $stdout;
 my $stderr;
@@ -119,8 +145,8 @@ sub script_runs {
   my $opt    = _options(\@_);
   my $unix   = shift @$args;
   my $path   = path( $unix );
-  my @libs   = map { "-I$_" } grep {!ref($_)} @INC;
-  my $cmd    = [ perl, @libs, $path, @$args ];
+  my $dir    = _preload_module();
+  my $cmd    = [ perl, "-I$dir", '-M__TEST_SCRIPT__', $path, @$args ];
      $stdout = '';
      $stderr = '';
   my $rv     = eval { run3( $cmd, $opt->{stdin}, $opt->{stdout}, $opt->{stderr} ) };
@@ -129,6 +155,8 @@ sub script_runs {
   my $signal = $? ? ($? & 127) : 0;
   my $ok     = !! ( $error eq '' and $rv and $exit == $opt->{exit} and $signal == $opt->{signal} );
 
+  File::Path::rmtree($dir);
+
   my $test = Test::Builder->new;
   $test->ok( $ok, $_[0] || "Script $unix runs" );
   $test->diag( "$exit - $stderr" ) unless $ok;
@@ -285,7 +313,7 @@ Test::Script - Basic cross-platform tests for scripts
 
 =head1 VERSION
 
-version 1.11_01
+version 1.11_02
 
 =head1 SYNOPSIS
 
